/*******************************************************************************

  wd_tst.c: main application for Lanner platform Watchdog/Bypass program

  Lanner Platform Miscellaneous Utility
  Copyright(c) 2015 Lanner Electronics Inc.
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer,
     without modification.
  2. Redistributions in binary form must reproduce at minimum a disclaimer
     similar to the "NO WARRANTY" disclaimer below ("Disclaimer") and any
     redistribution must be conditioned upon including a substantially
     similar Disclaimer requirement for further binary redistribution.
  3. Neither the names of the above-listed copyright holders nor the names
     of any contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  Alternatively, this software may be distributed under the terms of the
  GNU General Public License ("GPL") version 2 as published by the Free
  Software Foundation.

  NO WARRANTY
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF NONINFRINGEMENT, MERCHANTIBILITY
  AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY,
  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  THE POSSIBILITY OF SUCH DAMAGES.
*/
#include <stdio.h>
#ifdef DJGPP
/* For DOS djgpp */
#include <dos.h>
#include <inlines/pc.h>
#else
/* For Linux */
#include <sys/io.h>
#include <time.h>
#define inportb(x)      inb(x)
#define inportw(x)      inw(x)
#define inportl(x)      inl(x)
#define outportb(x, y)  outb(y, x)
#define outportw(x, y)  outw(y, x)
#define outportl(x, y)  outl(y, x)
#define delay(x)        usleep(x*100)
#define sleep(x)        usleep(x*1000000)
#endif

#define F81866_INDEX_PORT	0x4E
#define F81866_DATA_PORT	0x4F
#define BANK_REG		0x07
#define LDN06			0x06
#define LDN07			0x07

#define SHOW_COPYRIGHT_INFO_RR8000 \
    printf("****************************************************************\n");\
    printf("***          Lanner platform miscellaneous utility           ***\n");\
    printf("***  LEB-7233C   Watchdog/Bypass program   V1.0 2016-12-02  ***\n");\
    printf("****************************************************************\n")
	
//********************* Enter SIO *****************************************************
void EnterSioConfigMode(int INDEX)
{
	outportb(INDEX, 0x87);	// Enter Fintek 81866 configuration must do it twice per spec.
	outportb(INDEX, 0x87);
	return;
}
//********************* Exit SIO *****************************************************
void ExitSioConfigMode(int INDEX)
{
	outportb(INDEX, 0xAA);	// Exit Fintek 81866 configuration
}

//********************* Read SIO *****************************************************
unsigned char read_f81866_reg(int LDN, int reg)
{
	unsigned char tmp;
	EnterSioConfigMode(F81866_INDEX_PORT);	// EnterSioConfigMode
	outportb(F81866_INDEX_PORT, BANK_REG);	// Select to Logical Device Number(LDN) Register
	outportb(F81866_DATA_PORT, LDN);		// Write LDN to SIO
	outportb(F81866_INDEX_PORT, reg);		// Write register/index for access to
	tmp = inportb(F81866_DATA_PORT);		// Get data byte from SIO's register
	ExitSioConfigMode(F81866_INDEX_PORT);	// ExitSioConfigMode
	return tmp;
}

//********************* Write SIO *****************************************************
void write_f81866_reg(int LDN, int reg, int value)
{
	EnterSioConfigMode(F81866_INDEX_PORT);	// EnterSioConfigMode
	outportb(F81866_INDEX_PORT, BANK_REG);	// Select to Logical Device Number(LDN) Register
	outportb(F81866_DATA_PORT, LDN);		// Write LDN to SIO
	outportb(F81866_INDEX_PORT, reg);		// Write register/index for access to
	outportb(F81866_DATA_PORT, value);		// Write data byte into SIO's register
	ExitSioConfigMode(F81866_INDEX_PORT);	// ExitSioConfigMode
	return;
}

//********************* GPIO Init *****************************************************

void print_usage(char* argv0)
{
        printf("%s -swt xxx (Set Watchdog Timer 1-255 seconds and start to count-down)\n", argv0);
        printf("%s -stop (Stop Watchdog Timer)\n", argv0);
}


//---------------------------------------------------------------

void start_watchdog_timer(unsigned int watchdog_time)
{
	write_f81866_reg ( LDN07, 0xF6, watchdog_time );   	//LDN=7, CRF6, set watchdog timer time-out value
	write_f81866_reg ( LDN07, 0xF5, 0x20 );
}

void stop_watchdog_timer(void)
{
	write_f81866_reg ( LDN07, 0xFA, 0x10 );
}

void wd_gpio_init(void)
{
		  write_f81866_reg ( LDN07,  0x27, read_f81866_reg(LDN07, 0x27) | 0x04 );	//Multi Function Register Mapping For WDT
	      write_f81866_reg ( LDN07,  0x2C, read_f81866_reg(LDN07, 0x2C) & 0x00 );	//Multi Function Register Mapping For WDT	  
		  write_f81866_reg ( LDN07,  0x30, read_f81866_reg(LDN07, 0x30) | 0x01 );   // Enable GPIO
          write_f81866_reg ( LDN07,  0xFA, read_f81866_reg(LDN07, 0xFA) | 0x01 );   // GPIO A0 set as output		  
		  write_f81866_reg ( LDN07,  0xF5, read_f81866_reg(LDN07, 0xF5) | 0x40 );   // GPIOA Multi-function set as WDTO
          write_f81866_reg ( LDN07,  0xF6, 0x00 );
}

//---------------------------------------------------------------
int main( int argc, char** argv )
{
             iopl(3);

			SHOW_COPYRIGHT_INFO_RR8000;
			
             if ( argc < 2)
			 {
                  print_usage(argv[0]);
                  return -1;
             }

		     if ( !strcmp(argv[1], "-swt"))
			 {

			     wd_gpio_init();
			     unsigned int value;
                 unsigned int tmp;
                 
				 if (argc !=3)
				 {
                     printf("No timer input, program terminated\n");
                     return -1;
                 }
	                 tmp = atoi(argv[2]);
        	         if ( (tmp < 1) || (tmp > 255))
					 {
        	               printf("Wrong timer value, please input within (1~255)\n");
        	               return -1;
        	         }
		         start_watchdog_timer(tmp);

		         printf("Start Watchdog Timer:%d sec\n",atoi(argv[2]));
				 
	         }
/******** Stop watchdog timer ********************************************/
             else if ( !strcmp(argv[1], "-stop"))
			 {
		         wd_gpio_init();
	    	     stop_watchdog_timer();

	             printf("Stop Watchdog Timer....\n");
             }
	         else
			 {
		         printf("<<<Error: Unknown Parameter>>>\n");
                 print_usage(argv[0]);
		         return -1;
	         }

}

